ToraRange_Solver = {}

function ToraRange_Solver:Solve(initialPositions, distances, immovable)
	local positions = initialPositions

	-- Iteratively solve the positions
	for iter = 1, ToraRange_Options_SolverIterations do
		local newPositions = {}

		for idA, posA in pairs(positions) do
			newPositions[idA] = {x=posA.x, y=posA.y}

			-- Don't modify the own location (keep it at (0, 0))
			if not immovable[idA] then
				for idB, posB in pairs(positions) do
					if idA ~= idB and distances[idA] and distances[idA][idB] then
						local measuredDst = distances[idA][idB]

						-- Calculate the distance and the direction vector from A to B
						local dir = {
							x = posB.x - posA.x,
							y = posB.y - posA.y,
						}

						local dst = math.sqrt(dir.x * dir.x + dir.y * dir.y)

						if dst ~= 0 then
							dir.x = dir.x / dst
							dir.y = dir.y / dst

							-- Calculate force
							-- multiply by a half because the same is done for B later
							local force = (immovable[idB] and 1.0 or 0.5) * ToraRange_Options_SolverSpringConstant * (dst - measuredDst)

							-- Adjust new positions
							newPositions[idA].x = newPositions[idA].x + force * dir.x
							newPositions[idA].y = newPositions[idA].y + force * dir.y
						end
					end
				end
			end
		end

		positions = newPositions
	end

	return positions
end
