ToraRange_Util = {}
ToraRange_Util.RC = LibStub("LibRangeCheck-2.0")

function ToraRange_Util:GetGroupChannel()
	if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and IsInInstance() then
		return "INSTANCE_CHAT"
	end

	return "RAID"
end

-- Gets a players full name (Name-Realm) by his unit id
function ToraRange_Util:GetFullName(unitId)
	name, realm = UnitName(unitId)

	if not name then
		print("Name returned nil", unitId)
		return nil
	end

	if not realm or realm == "" then
		realm = string.gsub(GetRealmName(), "%s+", "")
	end

	return string.format("%s-%s", name, realm)
end

-- Gets a players full name (Name-Realm) by his group id
function ToraRange_Util:GetFullNameById(id)
	if not UnitInRaid("player") and not UnitInParty("player") then
		return nil
	end

	return self:GetFullName(ToraRange_Util:MakeGroupId(id))
end

-- Finds the group index of a player by his full name (Name-Realm)
function ToraRange_Util:GetPlayerGroupIndex(name)
	-- Use 5 for the local player if in a party
	if UnitInParty("player") and not UnitInRaid("player") and strlower(self:GetFullName("player")) == strlower(name) then
		return 5
	end

	if UnitInRaid("player") or UnitInParty("player") then
		for i = 1, self:GetGroupSize() do
			otherName = self:GetFullNameById(i)

			if otherName and strlower(name) == strlower(otherName) then
				return i
			end
		end
	end

	return nil
end

function ToraRange_Util:MakeGroupId(id)
	if UnitInRaid("player") then
		return "raid" .. id
	elseif UnitInParty("player") then
		if id == 5 then
			return "player"
		end

		return "party" .. id
	end

	return nil
end

function ToraRange_Util:GetLocalGroupId()
	return self:GetPlayerGroupIndex(self:GetFullName("player"))
end

function ToraRange_Util:IsPlayerRelevant(id)
	--local groupId = self:MakeGroupId(id)
	--return not UnitIsDeadOrGhost(groupId)
	return true
end

function ToraRange_Util:GetDistanceToUnit(unitId)
	-- Use LibRangeCheck if available
	if self.RC then
		minRange, maxRange = self.RC:GetRange(unitId)

		if minRange and maxRange then
			return (minRange + maxRange) / 2
		end
	else
		-- Check 40
		if UnitInRange(unitId) then
			-- Check 28
			if CheckInteractDistance(unitId, 1) then
				-- Check 8
				if CheckInteractDistance(unitId, 2) then
					-- Check 7
					if CheckInteractDistance(unitId, 3) then
						return (0+7)/2
					end

					return (7+8)/2
				end

				return (8+28)/2
			end

			return (28+40)/2
		end
	end

	return nil
end

function ToraRange_Util:GetGroupSize()
	return (UnitInRaid("player") and GetNumGroupMembers() or GetNumGroupMembers() - 1)
end

function ToraRange_Util:Bind(obj, func)
	return function(...)
		func(obj, ...)
	end
end
